/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Bound_h
#define Bound_h


#include "Primitives.h"
#include "Vector3f.h"

#include "hxa7241_graphics.h"


class bogus { bogus(); };   /// not functional, but if removed causes 'internal compiler error' in Render.cpp !!?




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Vector3f;
	using hxa7241::graphics::Transclip;
	using hxa7241::graphics::ClipFlags;


/**
 * object bound.<br/><br/>
 *
 * implemented as an axis aligned bounding box (AABB)<br/><br/>
 *
 * intersection algorithms:<br/>
 * <cite>ch2 p65, 'an introduction to ray tracing' - glassner (AP 89)</cite><br/>
 * <cite>'fast ray-box intersection' - woo (graphics gems, academic press, 1990)</cite><br/>
 * <cite>p.terdiman@codercorner.com</cite><br/>
 * <cite>schroeder (gdmag)</cite>
 *
 * @implementation
 * uncomment the chosen intersection method, comment out the others<br>
 * the one from 'an intro to ray tracing' was ~18% faster than the others, for one test.
 */
class Bound
{
/// standard object services -----------------------------------------------------------------------
public:
	         Bound();
	         Bound( const Vector3f& position,
	                const Vector3f& dimension );

	        ~Bound();
	         Bound( const Bound& );
	Bound& operator=( const Bound& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  setPosDim( const Vector3f& position, const Vector3f& dimension );
	        void  setMinMax( const Vector3f& min,      const Vector3f& max );


/// queries ----------------------------------------------------------------------------------------
public:
	        bool  getIntersection( const Ray& )                                               const;
	        bool  getClipping( const ClipFlags& enclosingClipFlags,
	                           const Transclip& transform,
	                           ClipFlags&       clipFlags )                                   const;

	        void  getMinMax( Vector3f& min, Vector3f& max )                                   const;
	        void  getCorners( Vector3f[8] )                                                   const;


/// fields -----------------------------------------------------------------------------------------
private:
	Vector3f position_m;
	Vector3f dimension_m;
};


}}




#endif//Bound_h
