/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef Perceptuum2_h
#define Perceptuum2_h


#include "Primitives.h"
#include "stdwin.h"
#include "ButtonBar.h"
#include "Win32ImagePaneSlideable.h"
#include <string>
#include <vector>

#include "perceptuum2_platform.h"




#include "perceptuum2_platform_ui.h"
namespace perceptuum2 { namespace platform_ui {
	using namespace hxa7241::primitives;
	using std::string;
	using std::vector;
	using perceptuum2::platform::RenderDesktopWin32;


class Perceptuum2
{
/// standard object services -----------------------------------------------------------------------
public:
	         Perceptuum2( HINSTANCE hInstance, HWND hWnd );

	        ~Perceptuum2();
private:
	         Perceptuum2( const Perceptuum2& );
	Perceptuum2& operator=( const Perceptuum2& );


/// commands ---------------------------------------------------------------------------------------
public:
	        void  setTonemapAdaptationLuminance( float );
	        void  stop();
	        void  stopWhenFrameDone();
	        void  stopForcibly();
	        void  unStop();
	        void  enableTimer( bool );

	        void  start( const string& filePathname, int nameOffset, int extOffset );
	        void  stop( int stopCode );
	        void  save( const string& filePathname, int extOffset );


/// queries ----------------------------------------------------------------------------------------
public:
	        float getTonemapAveLuminance()                                  const;
	        float getTonemapAdaptationLuminance()                           const;
	        bool  isThreadRunning()                                         const;
	        void  getSummaryNames( vector<string>& names )                  const;
	        void  getSummaries( vector<string>& summaries )                 const;
	        void  getStartFileNames( vector<string>& names )                const;
	        void  getFormattingNames( vector<string>& names )               const;

	static  const string& getAppName();
	static  const string& getAppVersion();
	static  const string& getAppNameAndVersion();
	static  const string& getAppTitle();
	static  const string& getAppBuildTime();
	static  const string& getLogFileName();


/// implementation ---------------------------------------------------------------------------------
private:
	        HWND     createStatusBar();
	        void     augmentSysMenu();
//	static  HPALETTE makePalette();

	        bool     getClientRectAroundImagePane( RECT& )                  const;
	        bool     getImagePaneRect( POINT& leftTop, POINT& widthHeight ) const;

	        void     makeNewRender( const string& projectName );
	        void     deleteRender();
	        RenderDesktopWin32* getRender()                                 const;
	        RenderDesktopWin32* getRenderThrow( const char* methodName )    const;

	        void     changeEnablementForRender();
	        bool     changeEnablementForStop();


/// message handlers -------------------------------------------------------------------------------
	//friend  LRESULT CALLBACK WindowProc( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );   /// wont work
public:
	        void     onSize( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam );
	        void     onPaint();
//	        LRESULT  onQueryNewPalette();
	        bool     onCommand( WPARAM );
	        bool     onCommandKey( WPARAM );
	        void     onBackground();
	        void     onLineUpdate  ( WPARAM pStatusLine, LPARAM lineNumber );
	        void     onFrameUpdate ( WPARAM pStatusLine, LPARAM shouldRepaint );
	        void     onRenderUpdate( WPARAM pStatusLine, LPARAM shouldRepaint );
	        void     onTimer();
	        bool     onClose();


/// fields -----------------------------------------------------------------------------------------
private:
	HINSTANCE hInstance_m;
	HWND      hWnd_m;
//	HPALETTE  hPalette_m;

	ButtonBar               buttonBar_m;
	HWND                    hWndStatusBar_m;
	Win32ImagePaneSlideable imagePane_m;

	RenderDesktopWin32*     pRender_m;

	static const int  TIMER_PERIOD_MILLISECS = 8000;
	static const UINT TIMER_ID               = 1;
	UINT              timerId_m;


public:
	static const TCHAR WND_CLASS_NAME[];
	static const char  CLASS_NAME[];

	static TCHAR APP_NAME[];
	static TCHAR APP_VERSION_MAIN[];
	static TCHAR APP_VERSION_POINT[];

	static const char  ERROR_MESSAGE1[];
	static const char  ERROR_MESSAGE2[];
	static const char  ERROR_MESSAGE3[];
	static const char  ERROR_MESSAGE4[];
};


}}




#endif//Perceptuum2_h
